#include "grid.h"
#include "head.h"

#include <stdlib.h>
#include <time.h>

#define NB_PASS 3
#define BIRTH_THRESHOLD 5
#define DEATH_THRESHOLD 3

struct square grid[GRID_WIDTH][GRID_HEIGHT];

SDL_Texture * square_texture[square_type_max][2];

void load_square_textures(SDL_Renderer * renderer) {
    for(enum square_type i = 0; i < square_type_max; i++) {
        switch(i) {
            case square_passable:
            case square_type_max:
                square_texture[i][0] = NULL;
                square_texture[i][1] = NULL;
                break;
            case square_rock:
                square_texture[i][0] = load_texture(renderer, "rock.bmp", NULL, NULL);
                square_texture[i][1] = load_texture(renderer, "rock_cracked.bmp", NULL, NULL);
                break;
            case square_cristal:
                square_texture[i][0] = load_texture(renderer, "cristal.bmp", NULL, NULL);
                square_texture[i][1] = load_texture(renderer, "cristal_cracked.bmp", NULL, NULL);
                break;
            case square_gem:
                square_texture[i][0] = load_texture(renderer, "gem.bmp", NULL, NULL);
                square_texture[i][1] = load_texture(renderer, "gem_cracked.bmp", NULL, NULL);
                break;
        }
    }
    gemicon = load_texture(renderer, "gemicon.bmp", NULL, NULL);
}

void setup_squares() {
    for(int x=0; x<GRID_WIDTH; x++) {
        for(int y=0; y<GRID_HEIGHT; y++) {
            grid[x][y].cracked = 0;
            grid[x][y].gem = 0;
            switch(grid[x][y].type) {
                case square_rock:
                    grid[x][y].integrity = 10;
                    break;
                case square_gem:
                    grid[x][y].integrity = 20;
                    grid[x][y].gem = 1;
                    break;
                case square_cristal:
                    grid[x][y].integrity = 50;
                    grid[x][y].gem = rand()%3+2;
                    break;
                case square_passable:
                    grid[x][y].integrity = 0;
                    break;
                case square_type_max:
                    break;
            }
        }
    }
}

int count_alive_neighbour(int x, int y) {
    int nb_alive = 0;
    for(int dx = -1; dx <= +1; dx++) {
        for(int dy = -1; dy <= +1; dy++) {
            if(!(dx == 0 && dy == 0)) {
                int nx = x + dx;
                int ny = y + dy;
                if(nx >= 0 && nx < GRID_WIDTH && ny >= 0 && ny < GRID_HEIGHT) {
                    if(grid[nx][ny].type == square_rock) {
                        nb_alive++;
                    }
                } else {
                    nb_alive++;
                }
            }
        }
    }
    return(nb_alive);
}

void init_grid(SDL_Renderer * renderer, int midripod_x, int midripod_y) {
    struct square next_grid[GRID_WIDTH][GRID_HEIGHT];
    load_square_textures(renderer);

    srand(time(NULL));
    for(int x=0; x<GRID_WIDTH; x++) {
        for(int y=0; y<GRID_HEIGHT; y++) {
            grid[x][y].type = rand()%2 ? square_passable : square_rock;
        }
    }

    for(int k=0; k<NB_PASS; k++) {

        for(int x=0; x<GRID_WIDTH; x++) {
            for(int y=0; y<GRID_HEIGHT; y++) {
                next_grid[x][y] = grid[x][y];
                int alive_neighbour = count_alive_neighbour(x, y);
                if(grid[x][y].type == square_rock) { // If Alive
                    if(alive_neighbour < DEATH_THRESHOLD) {
                        next_grid[x][y].type = square_passable; // Die
                    }
                } else { // If Dead
                    if(alive_neighbour > BIRTH_THRESHOLD) {
                        next_grid[x][y].type = square_rock; // Birth
                    }
                }
            }
        }

        for(int x=0; x<GRID_WIDTH; x++) {
            for(int y=0; y<GRID_HEIGHT; y++) {
                grid[x][y] = next_grid[x][y];
            }
        }

    }

    for(int nb_gem = rand()%3+8; nb_gem > 0; /* */) {
        int x = rand()%GRID_WIDTH;
        int y = rand()%GRID_HEIGHT;
        if(grid[x][y].type == square_rock) {
            grid[x][y].type = square_gem;
            nb_gem--;
        }
    }

    for(int nb_gem = rand()%2+2; nb_gem > 0; /* */) {
        int x = rand()%GRID_WIDTH;
        int y = rand()%GRID_HEIGHT;
        if(grid[x][y].type == square_passable) {
            grid[x][y].type = square_cristal;
            nb_gem--;
        }
    }

    setup_squares();
}

int is_passable(int x, int y) {
    return(grid[x/SQUARE_RATIO][y/SQUARE_RATIO].type == square_passable);
}

void grid_draw(SDL_Renderer * renderer) {
    SDL_Rect dst;
    dst.w = SQUARE_RATIO;
    dst.h = SQUARE_RATIO;

    for(int x=0; x<GRID_WIDTH; x++) {
        for(int y=0; y<GRID_HEIGHT; y++) {
            struct square * s = &(grid[x][y]);
            SDL_Texture * t = square_texture[s->type][s->cracked];
            dst.x = x*SQUARE_RATIO;
            dst.y = y*SQUARE_RATIO;
            switch(s->type) {
                case square_rock: SDL_RenderCopy(renderer, t, NULL, &dst); break;
                case square_gem: SDL_RenderCopy(renderer, t, NULL, &dst); break;
                case square_cristal: SDL_RenderCopy(renderer, t, NULL, &dst); break;
                case square_passable: break;
                case square_type_max: break;
            }
        }
    }
}

int square_dig(struct square * square, int drill_power) {
    int gem = 0;
    if(square->integrity > 0) {
        square->integrity -= drill_power;
        square->cracked = 1;
        if(square->integrity <= 0) {
            square->type = square_passable;
            gem += square->gem;
            square->gem = 0;
            square->integrity = 0;
        }
    }
    return(gem);
}

int grid_dig(int x, int y, int drill_power) {
    int gem = 0;
    for(int dx = -1; dx <= +1; dx++) {
        for(int dy = -1; dy <= +1; dy++) {
            int nx = (x + dx*SQUARE_RATIO/2) / SQUARE_RATIO;
            int ny = (y + dy*SQUARE_RATIO/2) / SQUARE_RATIO;
            if(nx >= 0 && nx < GRID_WIDTH && ny >= 0 && ny < GRID_HEIGHT) {
                gem += square_dig(&(grid[nx][ny]), drill_power);
            }
        }
    }
    return(gem);
}
