#include <stdlib.h>
#include <stdio.h>

#include "head.h"

void error(const char * msg) {
    fprintf(stderr, "%s: %s\n", msg, SDL_GetError());
    exit(EXIT_FAILURE);
}

SDL_Texture * load_texture(SDL_Renderer * renderer, const char * filename, int * w, int * h) {
    SDL_Texture * texture;
    SDL_Surface * surface = SDL_LoadBMP(filename);
    if(surface == NULL)
        error("Unable to load bmp file");
    texture = SDL_CreateTextureFromSurface(renderer, surface);
    if(texture == NULL)
        error("Unable to create texture from surface");
    if(w != NULL) *w = surface->w;
    if(h != NULL) *h = surface->h;
    SDL_FreeSurface(surface);
    return(texture);
}

int main(int argc, char ** argv) {
    SDL_Window * window;
    SDL_Renderer * renderer;
    SDL_Surface * icon;

    if(SDL_Init(SDL_INIT_VIDEO) != 0)
        error("Unable to init SDL");
    SDL_CreateWindowAndRenderer(0, 0, 0, &window, &renderer);
    if(window == NULL)
        error("Unable to open window");
    if(renderer == NULL)
        error("Unable to create renderer");

    SDL_SetWindowTitle(window, TITLE);
    SDL_SetWindowSize(window, WINDOW_WIDTH, WINDOW_HEIGHT);
    SDL_SetWindowPosition(window, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED);

    icon = SDL_LoadBMP(ICON_FILENAME);
    if(icon == NULL)
        error("Unable to open ICON_FILENAME file");
    SDL_SetWindowIcon(window, icon);

    loop(renderer);

    SDL_DestroyRenderer(renderer);
    SDL_DestroyWindow(window);
    SDL_Quit();

    return(EXIT_SUCCESS);
}
